@@ -127,13 +127,23 @@
 /*	int	var_smtputf8_enable
 /*	int	var_strict_smtputf8;
 /*	char	*var_smtputf8_autoclass;
+/*	int     var_compat_level;
+/*	char	*var_drop_hdrs;
 /*
 /*	void	mail_params_init()
 /*
 /*	const	char null_format_string[1];
+/*
+/*	int	warn_compat_break_app_dot_mydomain;
+/*	int	warn_compat_break_smtputf8_enable;
+/*	int	warn_compat_break_chroot;
+/*
+/*	int	warn_compat_break_relay_domains;
+/*	int	warn_compat_break_flush_domains;
+/*	int	warn_compat_break_mynetworks_style;
 /* DESCRIPTION
-/*	This module (actually the associated include file) define the names
-/*	and defaults of all mail configuration parameters.
+/*	This module (actually the associated include file) defines
+/*	the names and defaults of all mail configuration parameters.
 /*
 /*	mail_params_init() initializes the built-in parameters listed above.
 /*	These parameters are relied upon by library routines, so they are
@@ -143,6 +153,9 @@
 /*
 /*	null_format_string is a workaround for gcc compilers that complain
 /*	about empty or null format strings.
+/*
+/*	The warn_compat_XXX variables enable warnings for the use
+/*	of legacy default settings after an incompatible change.
 /* DIAGNOSTICS
 /*	Fatal errors: out of memory; null system or domain name.
 /* LICENSE
@@ -167,10 +180,6 @@
 #include <time.h>
 #include <ctype.h>
 
-#ifdef STRCASECMP_IN_STRINGS_H
-#include <strings.h>
-#endif
-
 /* Utility library. */
 
 #include <msg.h>
@@ -322,11 +331,24 @@ char   *var_dsn_filter;
 int     var_smtputf8_enable;
 int     var_strict_smtputf8;
 char   *var_smtputf8_autoclass;
+int     var_compat_level;
+char   *var_drop_hdrs;
 
 const char null_format_string[1] = "";
 
-DEFINE_DICT_LMDB_MAP_SIZE;
-DEFINE_DICT_DB_CACHE_SIZE;
+ /*
+  * Compatibility level 2.
+  */
+int     warn_compat_break_relay_domains;
+int     warn_compat_break_flush_domains;
+int     warn_compat_break_mynetworks_style;
+
+ /*
+  * Compatibility level 1.
+  */
+int     warn_compat_break_app_dot_mydomain;
+int     warn_compat_break_smtputf8_enable;
+int     warn_compat_break_chroot;
 
 /* check_myhostname - lookup hostname and validate */
 
@@ -528,10 +550,72 @@ static char *read_param_from_file(const char *path)
 
 #endif
 
+/* check_legacy_defaults - flag parameters that require safety-net logging */
+
+static void check_legacy_defaults(void)
+{
+
+    /*
+     * Basic idea: when an existing parameter default is changed, or a new
+     * parameter is introduced with incompatible default behavior, force
+     * Postfix to run with backwards-compatible default settings and log a
+     * warning when the backwards-compatible behavior is used.
+     * 
+     * Based on a review of Postfix logging the system administrator can decide
+     * whether or not to make backwards-compatible default settings permanent
+     * in main.cf or master.cf.
+     * 
+     * To turn off further warnings and deploy the new default settings, the
+     * system administrator should update the compatibility_level setting as
+     * recommended in the RELASE_NOTES file.
+     * 
+     * Each incompatible change has its own flag variable, instead of bit in a
+     * shared variable. We don't want to rip up code when we need more flag
+     * bits.
+     */
+
+    /*
+     * Look for specific parameters whose default changed when the
+     * compatibility level changed to 2.
+     */
+    if (var_compat_level < 2) {
+	if (mail_conf_lookup(VAR_RELAY_DOMAINS) == 0) {
+	    warn_compat_break_relay_domains = 1;
+	    if (mail_conf_lookup(VAR_FFLUSH_DOMAINS) == 0)
+		warn_compat_break_flush_domains = 1;
+	}
+	if (mail_conf_lookup(VAR_MYNETWORKS) == 0
+	    && mail_conf_lookup(VAR_MYNETWORKS_STYLE) == 0)
+	    warn_compat_break_mynetworks_style = 1;
+    }
+
+    /*
+     * Look for specific parameters whose default changed when the
+     * compatibility level changed from 0 to 1.
+     */
+    if (var_compat_level < 1) {
+	if (mail_conf_lookup(VAR_APP_DOT_MYDOMAIN) == 0)
+	    warn_compat_break_app_dot_mydomain = 1;
+
+	/*
+	 * Not: #ifndef NO_EAI. They must configure SMTPUTF8_ENABLE=no if a
+	 * warning message is logged, so that they don't suddenly start to
+	 * lose mail after Postfix is built with EAI support.
+	 */
+	if (mail_conf_lookup(VAR_SMTPUTF8_ENABLE) == 0)
+	    warn_compat_break_smtputf8_enable = 1;
+	warn_compat_break_chroot = 1;
+    }
+}
+
 /* mail_params_init - configure built-in parameters */
 
 void    mail_params_init()
 {
+    static const CONFIG_INT_TABLE first_int_defaults[] = {
+	VAR_COMPAT_LEVEL, DEF_COMPAT_LEVEL, &var_compat_level, 0, 0,
+	0,
+    };
     static const CONFIG_STR_TABLE first_str_defaults[] = {
 	/* $mail_version may appear in other parameters. */
 	VAR_MAIL_VERSION, DEF_MAIL_VERSION, &var_mail_version, 1, 0,
@@ -548,6 +632,11 @@ void    mail_params_init()
 	VAR_DAEMON_OPEN_FATAL, DEF_DAEMON_OPEN_FATAL, &var_daemon_open_fatal,
 	0,
     };
+    static const CONFIG_NBOOL_TABLE first_nbool_defaults[] = {
+	/* read and process the following before opening tables. */
+	VAR_SMTPUTF8_ENABLE, DEF_SMTPUTF8_ENABLE, &var_smtputf8_enable,
+	0,
+    };
     static const CONFIG_STR_FN_TABLE function_str_defaults[] = {
 	VAR_MYHOSTNAME, check_myhostname, &var_myhostname, 1, 0,
 	VAR_MYDOMAIN, check_mydomainname, &var_mydomain, 1, 0,
@@ -605,6 +694,7 @@ void    mail_params_init()
 	VAR_MULTI_WRAPPER, DEF_MULTI_WRAPPER, &var_multi_wrapper, 0, 0,
 	VAR_DSN_FILTER, DEF_DSN_FILTER, &var_dsn_filter, 0, 0,
 	VAR_SMTPUTF8_AUTOCLASS, DEF_SMTPUTF8_AUTOCLASS, &var_smtputf8_autoclass, 1, 0,
+	VAR_DROP_HDRS, DEF_DROP_HDRS, &var_drop_hdrs, 0, 0,
 	0,
     };
     static const CONFIG_STR_FN_TABLE function_str_defaults_2[] = {
@@ -666,12 +756,17 @@ void    mail_params_init()
 	VAR_CYRUS_SASL_AUTHZID, DEF_CYRUS_SASL_AUTHZID, &var_cyrus_sasl_authzid,
 	VAR_MULTI_ENABLE, DEF_MULTI_ENABLE, &var_multi_enable,
 	VAR_LONG_QUEUE_IDS, DEF_LONG_QUEUE_IDS, &var_long_queue_ids,
-	VAR_SMTPUTF8_ENABLE, DEF_SMTPUTF8_ENABLE, &var_smtputf8_enable,
 	VAR_STRICT_SMTPUTF8, DEF_STRICT_SMTPUTF8, &var_strict_smtputf8,
 	0,
     };
     const char *cp;
-    INET_PROTO_INFO *proto_info;
+
+    /*
+     * Extract compatibility level first, so that we can determine what
+     * parameters of interest are left at their legacy defaults.
+     */
+    get_mail_conf_int_table(first_int_defaults);
+    check_legacy_defaults();
 
     /*
      * Extract syslog_facility early, so that from here on all errors are
@@ -692,11 +787,28 @@ void    mail_params_init()
     if (var_daemon_open_fatal)
 	dict_allow_surrogate = 0;
 
+    /*
+     * Should we open tables with UTF8 support, or in the legacy 8-bit clean
+     * mode with ASCII-only casefolding?
+     */
+    get_mail_conf_nbool_table(first_nbool_defaults);
+
+    /*
+     * Report run-time versus compile-time discrepancies.
+     */
+#ifdef NO_EAI
+    if (var_smtputf8_enable)
+	msg_warn("%s is true, but EAI support is not compiled in",
+		 VAR_SMTPUTF8_ENABLE);
+    var_smtputf8_enable = 0;
+#endif
+    util_utf8_enable = var_smtputf8_enable;
+
     /*
      * What protocols should we attempt to support? The result is stored in
      * the global inet_proto_table variable.
      */
-    proto_info = inet_proto_init(VAR_INET_PROTOCOLS, var_inet_protocols);
+    (void) inet_proto_init(VAR_INET_PROTOCOLS, var_inet_protocols);
 
     /*
      * Variables whose defaults are determined at runtime. Some sites use
@@ -743,16 +855,6 @@ void    mail_params_init()
     dict_db_cache_size = var_db_read_buf;
     dict_lmdb_map_size = var_lmdb_map_size;
     inet_windowsize = var_inet_windowsize;
-    temp_utf8_kludge = var_smtputf8_enable;
-
-    /*
-     * Report run-time versus compile-time discrepancies.
-     */
-#ifdef NO_EAI
-    if (var_smtputf8_enable)
-	msg_warn("%s is true, but EAI support is not compiled in",
-		 VAR_SMTPUTF8_ENABLE);
-#endif
 
     /*
      * Variables whose defaults are determined at runtime, after other
@@ -791,18 +893,18 @@ void    mail_params_init()
     /*
      * I have seen this happen just too often.
      */
-    if (strcasecmp(var_myhostname, var_relayhost) == 0)
+    if (strcasecmp_utf8(var_myhostname, var_relayhost) == 0)
 	msg_fatal("%s and %s parameter settings must not be identical: %s",
 		  VAR_MYHOSTNAME, VAR_RELAYHOST, var_myhostname);
 
     /*
      * XXX These should be caught by a proper parameter parsing algorithm.
      */
-    if (var_myorigin[strcspn(var_myorigin, ", \t\r\n")])
+    if (var_myorigin[strcspn(var_myorigin, CHARS_COMMA_SP)])
 	msg_fatal("%s parameter setting must not contain multiple values: %s",
 		  VAR_MYORIGIN, var_myorigin);
 
-    if (var_relayhost[strcspn(var_relayhost, ", \t\r\n")])
+    if (var_relayhost[strcspn(var_relayhost, CHARS_COMMA_SP)])
 	msg_fatal("%s parameter setting must not contain multiple values: %s",
 		  VAR_RELAYHOST, var_relayhost);
 
