@@ -3,11 +3,11 @@
 
 /*++
 /* NAME
-/*      tls 3h
+/*	tls 3h
 /* SUMMARY
-/*      libtls internal interfaces
+/*	libtls internal interfaces
 /* SYNOPSIS
-/*      #include <tls.h>
+/*	#include <tls.h>
 /* DESCRIPTION
 /* .nf
 
@@ -23,6 +23,20 @@
   * 
   * XXX TLS_LEV_NOTFOUND no longer belongs in this list. The SMTP client will
   * have to use something else to report that policy table lookup failed.
+  * 
+  * The order of levels matters, but we hide most of the details in macros.
+  * 
+  * "dane" vs. "fingerprint", both must lie between "encrypt" and "verify".
+  * 
+  * - With "may" and higher, TLS is enabled.
+  * 
+  * - With "encrypt" and higher, TLS encryption must be applied.
+  * 
+  * - Strictly above "encrypt", the peer certificate must match.
+  * 
+  * - At "dane" and higher, the peer certificate must also be trusted. With
+  * "dane" the trust may be self-asserted, so we only log trust verification
+  * errors when TA associations are involved.
   */
 #define TLS_LEV_INVALID		-2	/* sentinel */
 #define TLS_LEV_NOTFOUND	-1	/* XXX not in policy table */
@@ -30,8 +44,17 @@
 #define TLS_LEV_MAY		1	/* wildcard */
 #define TLS_LEV_ENCRYPT		2	/* encrypted connection */
 #define TLS_LEV_FPRINT		3	/* "peer" CA-less verification */
-#define TLS_LEV_VERIFY		4	/* certificate verified */
-#define TLS_LEV_SECURE		5	/* "secure" verification */
+#define TLS_LEV_DANE		4	/* Opportunistic TLSA policy */
+#define TLS_LEV_DANE_ONLY	5	/* Required TLSA policy */
+#define TLS_LEV_VERIFY		6	/* certificate verified */
+#define TLS_LEV_SECURE		7	/* "secure" verification */
+
+#define TLS_REQUIRED(l)		((l) > TLS_LEV_MAY)
+#define TLS_MUST_MATCH(l)	((l) > TLS_LEV_ENCRYPT)
+#define TLS_MUST_TRUST(l)	((l) >= TLS_LEV_DANE)
+#define TLS_MUST_PKIX(l)	((l) >= TLS_LEV_VERIFY)
+#define TLS_OPPORTUNISTIC(l)	((l) == TLS_LEV_MAY || (l) == TLS_LEV_DANE)
+#define TLS_DANE_BASED(l)	((l) == TLS_LEV_DANE || (l) == TLS_LEV_DANE_ONLY)
 
 extern const NAME_CODE tls_level_table[];
 
@@ -52,8 +75,29 @@ extern const NAME_CODE tls_level_table[];
 #include <openssl/rand.h>
 #include <openssl/ssl.h>
 
-#if (OPENSSL_VERSION_NUMBER < 0x00905100L)
-#error "need OpenSSL version 0.9.5 or later"
+ /* Appease indent(1) */
+#define x509_stack_t STACK_OF(X509)
+#define x509_extension_stack_t STACK_OF(X509_EXTENSION)
+#define general_name_stack_t STACK_OF(GENERAL_NAME)
+#define ssl_cipher_stack_t STACK_OF(SSL_CIPHER)
+#define ssl_comp_stack_t STACK_OF(SSL_COMP)
+
+#if (OPENSSL_VERSION_NUMBER < 0x00090700f)
+#error "need OpenSSL version 0.9.7 or later"
+#endif
+
+/* SSL_CIPHER_get_name() got constified in 0.9.7g */
+#if OPENSSL_VERSION_NUMBER >= 0x0090707fL	/* constification */
+#define SSL_CIPHER_const const
+#else
+#define SSL_CIPHER_const
+#endif
+
+/* d2i_X509() got constified in 0.9.8a */
+#if OPENSSL_VERSION_NUMBER >= 0x0090801fL
+#define D2I_const const
+#else
+#define D2I_const
 #endif
 
  /*
@@ -63,6 +107,11 @@ extern const NAME_CODE tls_level_table[];
 #include <name_mask.h>
 #include <name_code.h>
 
+ /*
+  * TLS library.
+  */
+#include <dns.h>
+
  /*
   * Names of valid tlsmgr(8) session caches.
   */
@@ -70,6 +119,79 @@ extern const NAME_CODE tls_level_table[];
 #define TLS_MGR_SCACHE_SMTP	"smtp"
 #define TLS_MGR_SCACHE_LMTP	"lmtp"
 
+ /*
+  * RFC 6698 DANE
+  */
+#define TLS_DANE_TA	0		/* Match trust-anchor digests */
+#define TLS_DANE_EE	1		/* Match end-entity digests */
+
+#define TLS_DANE_CERT	0		/* Match the certificate digest */
+#define TLS_DANE_PKEY	1		/* Match the public key digest */
+
+#define TLS_DANE_FLAG_NORRS	(1<<0)	/* Nothing found in DNS */
+#define TLS_DANE_FLAG_EMPTY	(1<<1)	/* Nothing usable found in DNS */
+#define TLS_DANE_FLAG_ERROR	(1<<2)	/* TLSA record lookup error */
+
+#define tls_dane_unusable(dane)	((dane)->flags & TLS_DANE_FLAG_EMPTY)
+#define tls_dane_notfound(dane)	((dane)->flags & TLS_DANE_FLAG_NORRS)
+
+#define TLS_DANE_CACHE_TTL_MIN 1	/* A lot can happen in ~2 seconds */
+#define TLS_DANE_CACHE_TTL_MAX 100	/* Comparable to max_idle */
+
+ /*
+  * Certificate and public key digests (typically from TLSA RRs), grouped by
+  * algorithm.
+  */
+typedef struct TLS_TLSA {
+    char   *mdalg;			/* Algorithm for this digest list */
+    ARGV   *certs;			/* Complete certificate digests */
+    ARGV   *pkeys;			/* SubjectPublicKeyInfo digests */
+    struct TLS_TLSA *next;		/* Chain to next algorithm */
+} TLS_TLSA;
+
+ /*
+  * Linked list of full X509 trust-anchor certs.
+  */
+typedef struct TLS_CERTS {
+    X509   *cert;
+    struct TLS_CERTS *next;
+} TLS_CERTS;
+
+ /*
+  * Linked list of full EVP_PKEY trust-anchor public keys.
+  */
+typedef struct TLS_PKEYS {
+    EVP_PKEY *pkey;
+    struct TLS_PKEYS *next;
+} TLS_PKEYS;
+
+typedef struct TLS_DANE {
+    TLS_TLSA *ta;			/* Trust-anchor cert/pubkey digests */
+    TLS_TLSA *ee;			/* End-entity cert/pubkey digests */
+    TLS_CERTS *certs;			/* Full trust-anchor certificates */
+    TLS_PKEYS *pkeys;			/* Full trust-anchor public keys */
+    char   *base_domain;		/* Base domain of TLSA RRset */
+    int     flags;			/* Conflate cert and pkey digests */
+    time_t  expires;			/* Expiration time of this record */
+    int     refs;			/* Reference count */
+} TLS_DANE;
+
+#define TLS_DANE_HASTA(d)	((d) ? (d)->ta : 0)
+#define TLS_DANE_HASEE(d)	((d) ? (d)->ee : 0)
+
+ /*
+  * tls_dane.c
+  */
+extern int tls_dane_avail(void);
+extern void tls_dane_flush(void);
+extern void tls_dane_verbose(int);
+extern TLS_DANE *tls_dane_alloc(void);
+extern void tls_dane_add_ee_digests(TLS_DANE *, const char *, const char *,
+				            const char *);
+extern void tls_dane_free(TLS_DANE *);
+extern TLS_DANE *tls_dane_resolve(unsigned, const char *, DNS_RR *, int);
+extern int tls_dane_load_trustfile(TLS_DANE *, const char *);
+
  /*
   * TLS session context, also used by the VSTREAM call-back routines for SMTP
   * input/output, and by OpenSSL call-back routines for key verification.
@@ -82,7 +204,7 @@ typedef struct {
     /* Public, read-only. */
     char   *peer_CN;			/* Peer Common Name */
     char   *issuer_CN;			/* Issuer Common Name */
-    char   *peer_fingerprint;		/* ASCII fingerprint */
+    char   *peer_cert_fprint;		/* ASCII certificate fingerprint */
     char   *peer_pkey_fprint;		/* ASCII public key fingerprint */
     int     peer_status;		/* Certificate and match status */
     const char *protocol;
@@ -92,14 +214,23 @@ typedef struct {
     /* Private. */
     SSL    *con;
     char   *cache_type;			/* tlsmgr(8) cache type if enabled */
+    int     ticketed;			/* Session ticket issued */
     char   *serverid;			/* unique server identifier */
     char   *namaddr;			/* nam[addr] for logging */
     int     log_mask;			/* What to log */
     int     session_reused;		/* this session was reused */
     int     am_server;			/* Are we an SSL server or client? */
+    const char *mdalg;			/* default message digest algorithm */
     /* Built-in vs external SSL_accept/read/write/shutdown support. */
-    char   *fpt_dgst;			/* Certificate fingerprint digest */
     VSTREAM *stream;			/* Blocking-mode SMTP session */
+    /* RFC 6698 DANE trust input and verification state */
+    const TLS_DANE *dane;		/* DANE TLSA digests */
+    int     errordepth;			/* Chain depth of error cert */
+    int     tadepth;			/* Chain depth of trust anchor */
+    int     errorcode;			/* First error at error depth */
+    X509   *errorcert;			/* Error certificate closest to leaf */
+    x509_stack_t *untrusted;		/* Certificate chain fodder */
+    x509_stack_t *trusted;		/* Internal root CA list */
 } TLS_SESS_STATE;
 
  /*
@@ -156,8 +287,9 @@ struct TLS_APPL_STATE {
 };
 
  /*
-  * tls_misc.c One time finalization of application context.
+  * tls_misc.c Application-context update and disposal.
   */
+extern void tls_update_app_logmask(TLS_APPL_STATE *, int);
 extern void tls_free_app_context(TLS_APPL_STATE *);
 
  /*
@@ -170,26 +302,78 @@ extern void tls_param_init(void);
   * Protocol selection.
   */
 #define TLS_PROTOCOL_INVALID	(~0)	/* All protocol bits masked */
+
+#ifdef SSL_TXT_SSLV2
 #define TLS_PROTOCOL_SSLv2	(1<<0)	/* SSLv2 */
+#else
+#define SSL_TXT_SSLV2		"SSLv2"
+#define TLS_PROTOCOL_SSLv2	0	/* Unknown */
+#undef  SSL_OP_NO_SSLv2
+#define SSL_OP_NO_SSLv2		0L	/* Noop */
+#endif
+
+#ifdef SSL_TXT_SSLV3
 #define TLS_PROTOCOL_SSLv3	(1<<1)	/* SSLv3 */
+#else
+#define SSL_TXT_SSLV3		"SSLv3"
+#define TLS_PROTOCOL_SSLv3	0	/* Unknown */
+#undef  SSL_OP_NO_SSLv3
+#define SSL_OP_NO_SSLv3		0L	/* Noop */
+#endif
+
+#ifdef SSL_TXT_TLSV1
 #define TLS_PROTOCOL_TLSv1	(1<<2)	/* TLSv1 */
+#else
+#define SSL_TXT_TLSV1		"TLSv1"
+#define TLS_PROTOCOL_TLSv1	0	/* Unknown */
+#undef  SSL_OP_NO_TLSv1
+#define SSL_OP_NO_TLSv1		0L	/* Noop */
+#endif
+
 #ifdef SSL_TXT_TLSV1_1
 #define TLS_PROTOCOL_TLSv1_1	(1<<3)	/* TLSv1_1 */
 #else
+#define SSL_TXT_TLSV1_1		"TLSv1.1"
 #define TLS_PROTOCOL_TLSv1_1	0	/* Unknown */
 #undef  SSL_OP_NO_TLSv1_1
 #define SSL_OP_NO_TLSv1_1	0L	/* Noop */
 #endif
+
 #ifdef SSL_TXT_TLSV1_2
 #define TLS_PROTOCOL_TLSv1_2	(1<<4)	/* TLSv1_2 */
 #else
+#define SSL_TXT_TLSV1_2		"TLSv1.2"
 #define TLS_PROTOCOL_TLSv1_2	0	/* Unknown */
 #undef  SSL_OP_NO_TLSv1_2
 #define SSL_OP_NO_TLSv1_2	0L	/* Noop */
 #endif
-#define TLS_KNOWN_PROTOCOLS	\
+
+#ifdef SSL_TXT_TLSV1_3
+#define TLS_PROTOCOL_TLSv1_3	(1<<5)	/* TLSv1_3 */
+#else
+#define SSL_TXT_TLSV1_3		"TLSv1.3"
+#define TLS_PROTOCOL_TLSv1_3	0	/* Unknown */
+#undef  SSL_OP_NO_TLSv1_3
+#define SSL_OP_NO_TLSv1_3	0L	/* Noop */
+#endif
+
+#define TLS_KNOWN_PROTOCOLS \
 	( TLS_PROTOCOL_SSLv2 | TLS_PROTOCOL_SSLv3 | TLS_PROTOCOL_TLSv1 \
 	   | TLS_PROTOCOL_TLSv1_1 | TLS_PROTOCOL_TLSv1_2 )
+#define TLS_SSL_OP_PROTOMASK(m) \
+	    ((((m) & TLS_PROTOCOL_SSLv2) ? SSL_OP_NO_SSLv2 : 0L) \
+	     | (((m) & TLS_PROTOCOL_SSLv3) ? SSL_OP_NO_SSLv3 : 0L) \
+	     | (((m) & TLS_PROTOCOL_TLSv1) ? SSL_OP_NO_TLSv1 : 0L) \
+	     | (((m) & TLS_PROTOCOL_TLSv1_1) ? SSL_OP_NO_TLSv1_1 : 0L) \
+	     | (((m) & TLS_PROTOCOL_TLSv1_2) ? SSL_OP_NO_TLSv1_2 : 0L) \
+	     | (((m) & TLS_PROTOCOL_TLSv1_3) ? SSL_OP_NO_TLSv1_3 : 0L))
+
+/*
+ * SSL options that are managed via dedicated Postfix features, rather than
+ * just exposed via hex codes or named elements of tls_ssl_options.
+ */
+#define TLS_SSL_OP_MANAGED_BITS \
+	(SSL_OP_CIPHER_SERVER_PREFERENCE | TLS_SSL_OP_PROTOMASK(~0))
 
 extern int tls_protocol_mask(const char *);
 
@@ -234,7 +418,7 @@ typedef struct {
     const char *eckey_file;
     const char *CAfile;
     const char *CApath;
-    const char *fpt_dgst;		/* Fingerprint digest algorithm */
+    const char *mdalg;			/* default message digest algorithm */
 } TLS_CLIENT_INIT_PROPS;
 
 typedef struct {
@@ -246,11 +430,13 @@ typedef struct {
     const char *host;			/* MX hostname */
     const char *namaddr;		/* nam[addr] for logging */
     const char *serverid;		/* Session cache key */
+    const char *helo;			/* Server name from EHLO response */
     const char *protocols;		/* Enabled protocols */
     const char *cipher_grade;		/* Minimum cipher grade */
     const char *cipher_exclusions;	/* Ciphers to exclude */
     const ARGV *matchargv;		/* Cert match patterns */
-    const char *fpt_dgst;		/* Fingerprint digest algorithm */
+    const char *mdalg;			/* default message digest algorithm */
+    const TLS_DANE *dane;		/* RFC 6698 verification */
 } TLS_CLIENT_START_PROPS;
 
 extern TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *);
@@ -267,11 +453,11 @@ extern TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *);
     ((props)->a12), ((props)->a13), (props)))
 
 #define TLS_CLIENT_START(props, a1, a2, a3, a4, a5, a6, a7, a8, a9, \
-    a10, a11, a12, a13) \
+    a10, a11, a12, a13, a14, a15) \
     tls_client_start((((props)->a1), ((props)->a2), ((props)->a3), \
     ((props)->a4), ((props)->a5), ((props)->a6), ((props)->a7), \
     ((props)->a8), ((props)->a9), ((props)->a10), ((props)->a11), \
-    ((props)->a12), ((props)->a13), (props)))
+    ((props)->a12), ((props)->a13), ((props)->a14), ((props)->a15), (props)))
 
  /*
   * tls_server.c
@@ -281,7 +467,6 @@ typedef struct {
     const char *log_level;
     int     verifydepth;
     const char *cache_type;
-    long    scache_timeout;
     int     set_sessid;
     const char *cert_file;
     const char *key_file;
@@ -296,7 +481,7 @@ typedef struct {
     const char *dh1024_param_file;
     const char *dh512_param_file;
     int     ask_ccert;
-    const char *fpt_dgst;		/* Fingerprint digest algorithm */
+    const char *mdalg;			/* default message digest algorithm */
 } TLS_SERVER_INIT_PROPS;
 
 typedef struct {
@@ -309,7 +494,7 @@ typedef struct {
     const char *namaddr;		/* Client nam[addr] for logging */
     const char *cipher_grade;
     const char *cipher_exclusions;
-    const char *fpt_dgst;		/* Fingerprint digest algorithm */
+    const char *mdalg;			/* default message digest algorithm */
 } TLS_SERVER_START_PROPS;
 
 extern TLS_APPL_STATE *tls_server_init(const TLS_SERVER_INIT_PROPS *);
@@ -320,13 +505,12 @@ extern TLS_SESS_STATE *tls_server_post_accept(TLS_SESS_STATE *);
 	tls_session_stop(ctx, (stream), (timeout), (failure), (TLScontext))
 
 #define TLS_SERVER_INIT(props, a1, a2, a3, a4, a5, a6, a7, a8, a9, \
-    a10, a11, a12, a13, a14, a15, a16, a17, a18, a19, a20) \
+    a10, a11, a12, a13, a14, a15, a16, a17, a18, a19) \
     tls_server_init((((props)->a1), ((props)->a2), ((props)->a3), \
     ((props)->a4), ((props)->a5), ((props)->a6), ((props)->a7), \
     ((props)->a8), ((props)->a9), ((props)->a10), ((props)->a11), \
     ((props)->a12), ((props)->a13), ((props)->a14), ((props)->a15), \
-    ((props)->a16), ((props)->a17), ((props)->a18), ((props)->a19), \
-    ((props)->a20), (props)))
+    ((props)->a16), ((props)->a17), ((props)->a18), ((props)->a19), (props)))
 
 #define TLS_SERVER_START(props, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10) \
     tls_server_start((((props)->a1), ((props)->a2), ((props)->a3), \
@@ -397,9 +581,24 @@ extern RSA *tls_tmp_rsa_cb(SSL *, int, int);
 extern char *tls_peer_CN(X509 *, const TLS_SESS_STATE *);
 extern char *tls_issuer_CN(X509 *, const TLS_SESS_STATE *);
 extern const char *tls_dns_name(const GENERAL_NAME *, const TLS_SESS_STATE *);
-extern char *tls_fingerprint(X509 *, const char *);
-extern char *tls_pkey_fprint(X509 *, const char *);
 extern int tls_verify_certificate_callback(int, X509_STORE_CTX *);
+extern void tls_log_verify_error(TLS_SESS_STATE *);
+
+ /*
+  * tls_dane.c
+  */
+extern int tls_dane_match(TLS_SESS_STATE *, int, X509 *, int);
+extern void tls_dane_set_callback(SSL_CTX *, TLS_SESS_STATE *);
+
+ /*
+  * tls_fprint.c
+  */
+extern char *tls_digest_encode(const unsigned char *, int);
+extern char *tls_data_fprint(const char *, int, const char *);
+extern char *tls_cert_fprint(X509 *, const char *);
+extern char *tls_pkey_fprint(X509 *, const char *);
+extern char *tls_serverid_digest(const TLS_CLIENT_START_PROPS *, long,
+				         const char *);
 
  /*
   * tls_certkey.c
@@ -423,6 +622,7 @@ extern long tls_bug_bits(void);
 extern void tls_print_errors(void);
 extern void tls_info_callback(const SSL *, int, int);
 extern long tls_bio_dump_cb(BIO *, int, const char *, int, long, long);
+extern int tls_validate_digest(const char *);
 
  /*
   * tls_seed.c
@@ -435,12 +635,12 @@ extern int tls_ext_seed(int);
 /* LICENSE
 /* .ad
 /* .fi
-/*      The Secure Mailer license must be distributed with this software.
+/*	The Secure Mailer license must be distributed with this software.
 /* AUTHOR(S)
-/*      Wietse Venema
-/*      IBM T.J. Watson Research
-/*      P.O. Box 704
-/*      Yorktown Heights, NY 10598, USA
+/*	Wietse Venema
+/*	IBM T.J. Watson Research
+/*	P.O. Box 704
+/*	Yorktown Heights, NY 10598, USA
 /*
 /*	Victor Duchovni
 /*	Morgan Stanley
