@@ -84,10 +84,10 @@
 /*	format.
 /* .IP "\fBtlsproxy_tls_dh1024_param_file ($smtpd_tls_dh1024_param_file)\fR"
 /*	File with DH parameters that the Postfix \fBtlsproxy\fR(8) server
-/*	should use with EDH ciphers.
+/*	should use with non-export EDH ciphers.
 /* .IP "\fBtlsproxy_tls_dh512_param_file ($smtpd_tls_dh512_param_file)\fR"
 /*	File with DH parameters that the Postfix \fBtlsproxy\fR(8) server
-/*	should use with EDH ciphers.
+/*	should use with export-grade EDH ciphers.
 /* .IP "\fBtlsproxy_tls_dkey_file ($smtpd_tls_dkey_file)\fR"
 /*	File with the Postfix \fBtlsproxy\fR(8) server DSA private key in PEM
 /*	format.
@@ -132,9 +132,10 @@
 /*	The SMTP TLS security level for the Postfix \fBtlsproxy\fR(8) server;
 /*	when a non-empty value is specified, this overrides the obsolete
 /*	parameters smtpd_use_tls and smtpd_enforce_tls.
-/* .IP "\fBtlsproxy_tls_session_cache_timeout ($smtpd_tls_session_cache_timeout)\fR"
-/*	The expiration time of Postfix \fBtlsproxy\fR(8) server TLS session
-/*	cache information.
+/* .PP
+/*	Available in Postfix version 2.11 and later:
+/* .IP "\fBtlsmgr_service_name (tlsmgr)\fR"
+/*	The name of the \fBtlsmgr\fR(8) service entry in master.cf.
 /* OBSOLETE STARTTLS SUPPORT CONTROLS
 /* .ad
 /* .fi
@@ -191,6 +192,11 @@
   * System library.
   */
 #include <sys_defs.h>
+#include <errno.h>
+
+#ifdef STRCASECMP_IN_STRINGS_H
+#include <strings.h>
+#endif
 
  /*
   * Utility library.
@@ -233,7 +239,6 @@
   */
 int     var_smtpd_tls_ccert_vd;
 char   *var_smtpd_tls_loglevel;
-int     var_smtpd_tls_scache_timeout;
 bool    var_smtpd_use_tls;
 bool    var_smtpd_enforce_tls;
 bool    var_smtpd_tls_ask_ccert;
@@ -262,7 +267,6 @@ char   *var_smtpd_tls_level;
 
 int     var_tlsp_tls_ccert_vd;
 char   *var_tlsp_tls_loglevel;
-int     var_tlsp_tls_scache_timeout;
 bool    var_tlsp_use_tls;
 bool    var_tlsp_enforce_tls;
 bool    var_tlsp_tls_ask_ccert;
@@ -332,7 +336,7 @@ static int ask_client_cert;
   * become part of the TLS library.
   */
 
-static void tlsp_ciphertext_event(int, char *);
+static void tlsp_ciphertext_event(int, void *);
 
 #define TLSP_INIT_TIMEOUT	100
 
@@ -389,7 +393,7 @@ static int tlsp_eval_tls_error(TLSP_STATE *state, int err)
     case SSL_ERROR_NONE:
 	if (state->ssl_last_err != SSL_ERROR_NONE) {
 	    event_disable_readwrite(ciphertext_fd);
-	    event_request_timer(tlsp_ciphertext_event, (char *) state,
+	    event_request_timer(tlsp_ciphertext_event, (void *) state,
 				state->timeout);
 	    state->ssl_last_err = SSL_ERROR_NONE;
 	}
@@ -404,10 +408,10 @@ static int tlsp_eval_tls_error(TLSP_STATE *state, int err)
 	    event_disable_readwrite(ciphertext_fd);
 	if (state->ssl_last_err != SSL_ERROR_WANT_WRITE) {
 	    event_enable_write(ciphertext_fd, tlsp_ciphertext_event,
-			       (char *) state);
+			       (void *) state);
 	    state->ssl_last_err = SSL_ERROR_WANT_WRITE;
 	}
-	event_request_timer(tlsp_ciphertext_event, (char *) state,
+	event_request_timer(tlsp_ciphertext_event, (void *) state,
 			    state->timeout);
 	return (0);
 
@@ -420,10 +424,10 @@ static int tlsp_eval_tls_error(TLSP_STATE *state, int err)
 	    event_disable_readwrite(ciphertext_fd);
 	if (state->ssl_last_err != SSL_ERROR_WANT_READ) {
 	    event_enable_read(ciphertext_fd, tlsp_ciphertext_event,
-			      (char *) state);
+			      (void *) state);
 	    state->ssl_last_err = SSL_ERROR_WANT_READ;
 	}
-	event_request_timer(tlsp_ciphertext_event, (char *) state,
+	event_request_timer(tlsp_ciphertext_event, (void *) state,
 			    state->timeout);
 	return (0);
 
@@ -471,8 +475,9 @@ static void tlsp_strategy(TLSP_STATE *state)
 	}
 	if ((state->req_flags & TLS_PROXY_FLAG_SEND_CONTEXT) != 0
 	    && (attr_print(state->plaintext_stream, ATTR_FLAG_NONE,
-			   ATTR_TYPE_FUNC, tls_proxy_context_print,
-			   (char *) state->tls_context, ATTR_TYPE_END) != 0
+			   SEND_ATTR_FUNC(tls_proxy_context_print,
+					  (void *) state->tls_context),
+			   ATTR_TYPE_END) != 0
 		|| vstream_fflush(state->plaintext_stream) != 0)) {
 	    msg_warn("cannot send TLS context: %m");
 	    tlsp_state_free(state);
@@ -601,7 +606,7 @@ static void tlsp_strategy(TLSP_STATE *state)
 
 /* tlsp_plaintext_event - plaintext was read/written */
 
-static void tlsp_plaintext_event(int event, char *context)
+static void tlsp_plaintext_event(int event, void *context)
 {
     TLSP_STATE *state = (TLSP_STATE *) context;
 
@@ -624,7 +629,7 @@ static void tlsp_plaintext_event(int event, char *context)
 
 /* tlsp_ciphertext_event - ciphertext is ready to read/write */
 
-static void tlsp_ciphertext_event(int event, char *context)
+static void tlsp_ciphertext_event(int event, void *context)
 {
     TLSP_STATE *state = (TLSP_STATE *) context;
 
@@ -698,7 +703,7 @@ static void tlsp_start_tls(TLSP_STATE *state)
 			 namaddr = state->remote_endpt,
 			 cipher_grade = cipher_grade,
 			 cipher_exclusions = STR(cipher_exclusions),
-			 fpt_dgst = var_tlsp_tls_fpt_dgst);
+			 mdalg = var_tlsp_tls_fpt_dgst);
 
     if (state->tls_context == 0) {
 	tlsp_state_free(state);
@@ -719,7 +724,7 @@ static void tlsp_start_tls(TLSP_STATE *state)
 
 /* tlsp_get_fd_event - receive final postscreen(8) hand-off information */
 
-static void tlsp_get_fd_event(int event, char *context)
+static void tlsp_get_fd_event(int event, void *context)
 {
     const char *myname = "tlsp_get_fd_event";
     TLSP_STATE *state = (TLSP_STATE *) context;
@@ -732,7 +737,9 @@ static void tlsp_get_fd_event(int event, char *context)
      */
     event_disable_readwrite(plaintext_fd);
     if (event != EVENT_TIME)
-	event_cancel_timer(tlsp_get_fd_event, (char *) state);
+	event_cancel_timer(tlsp_get_fd_event, (void *) state);
+    else
+	errno = ETIMEDOUT;
 
     /*
      * Initialize plaintext-related session state.  Once we have this behind
@@ -754,7 +761,7 @@ static void tlsp_get_fd_event(int event, char *context)
     state->plaintext_buf = nbbio_create(plaintext_fd,
 					VSTREAM_BUFSIZE, "postscreen",
 					tlsp_plaintext_event,
-					(char *) state);
+					(void *) state);
 
     /*
      * Perform the TLS layer before-handshake initialization. We perform the
@@ -770,7 +777,7 @@ static void tlsp_get_fd_event(int event, char *context)
 
 /* tlsp_get_request_event - receive initial postscreen(8) hand-off info */
 
-static void tlsp_get_request_event(int event, char *context)
+static void tlsp_get_request_event(int event, void *context)
 {
     const char *myname = "tlsp_get_request_event";
     TLSP_STATE *state = (TLSP_STATE *) context;
@@ -796,7 +803,9 @@ static void tlsp_get_request_event(int event, char *context)
      * and redefine read events on success.
      */
     if (event != EVENT_TIME)
-	event_cancel_timer(tlsp_get_request_event, (char *) state);
+	event_cancel_timer(tlsp_get_request_event, (void *) state);
+    else
+	errno = ETIMEDOUT;
 
     /*
      * We must send some data, after receiving the request attributes and
@@ -805,10 +814,10 @@ static void tlsp_get_request_event(int event, char *context)
      */
     if (event != EVENT_READ
 	|| attr_scan(plaintext_stream, ATTR_FLAG_STRICT,
-		     ATTR_TYPE_STR, MAIL_ATTR_REMOTE_ENDPT, remote_endpt,
-		     ATTR_TYPE_INT, MAIL_ATTR_FLAGS, &req_flags,
-		     ATTR_TYPE_INT, MAIL_ATTR_TIMEOUT, &timeout,
-		     ATTR_TYPE_STR, MAIL_ATTR_SERVER_ID, server_id,
+		     RECV_ATTR_STR(MAIL_ATTR_REMOTE_ENDPT, remote_endpt),
+		     RECV_ATTR_INT(MAIL_ATTR_FLAGS, &req_flags),
+		     RECV_ATTR_INT(MAIL_ATTR_TIMEOUT, &timeout),
+		     RECV_ATTR_STR(MAIL_ATTR_SERVER_ID, server_id),
 		     ATTR_TYPE_END) != 4) {
 	msg_warn("%s: receive request attributes: %m", myname);
 	event_disable_readwrite(plaintext_fd);
@@ -823,7 +832,7 @@ static void tlsp_get_request_event(int event, char *context)
     ready = ((req_flags & TLS_PROXY_FLAG_ROLE_SERVER) != 0
 	     && tlsp_server_ctx != 0);
     if (attr_print(plaintext_stream, ATTR_FLAG_NONE,
-		   ATTR_TYPE_INT, MAIL_ATTR_STATUS, ready,
+		   SEND_ATTR_INT(MAIL_ATTR_STATUS, ready),
 		   ATTR_TYPE_END) != 0
 	|| vstream_fflush(plaintext_stream) != 0
 	|| ready == 0) {
@@ -848,8 +857,8 @@ static void tlsp_get_request_event(int event, char *context)
 		 "(bogus_direction)", state->remote_endpt);
 	state->req_flags = req_flags;
 	state->timeout = timeout + 10;		/* XXX */
-	event_enable_read(plaintext_fd, tlsp_get_fd_event, (char *) state);
-	event_request_timer(tlsp_get_fd_event, (char *) state,
+	event_enable_read(plaintext_fd, tlsp_get_fd_event, (void *) state);
+	event_request_timer(tlsp_get_fd_event, (void *) state,
 			    TLSP_INIT_TIMEOUT);
 	return;
     }
@@ -878,16 +887,16 @@ static void tlsp_service(VSTREAM *plaintext_stream,
      */
     non_blocking(plaintext_fd, NON_BLOCKING);
     vstream_control(plaintext_stream,
-		    VSTREAM_CTL_PATH, "plaintext",
-		    VSTREAM_CTL_TIMEOUT, 5,
-		    VSTREAM_CTL_END);
+		    CA_VSTREAM_CTL_PATH("plaintext"),
+		    CA_VSTREAM_CTL_TIMEOUT(5),
+		    CA_VSTREAM_CTL_END);
 
     /*
      * Receive postscreen's remote SMTP client address/port and socket.
      */
     state = tlsp_state_create(service, plaintext_stream);
-    event_enable_read(plaintext_fd, tlsp_get_request_event, (char *) state);
-    event_request_timer(tlsp_get_request_event, (char *) state,
+    event_enable_read(plaintext_fd, tlsp_get_request_event, (void *) state);
+    event_request_timer(tlsp_get_request_event, (void *) state,
 			TLSP_INIT_TIMEOUT);
 }
 
@@ -974,7 +983,6 @@ static void pre_jail_init(char *unused_name, char **unused_argv)
 			    log_level = var_tlsp_tls_loglevel,
 			    verifydepth = var_tlsp_tls_ccert_vd,
 			    cache_type = TLS_MGR_SCACHE_SMTPD,
-			    scache_timeout = var_tlsp_tls_scache_timeout,
 			    set_sessid = var_tlsp_tls_set_sessid,
 			    cert_file = cert_file,
 			    key_file = var_tlsp_tls_key_file,
@@ -993,7 +1001,7 @@ static void pre_jail_init(char *unused_name, char **unused_argv)
 			    var_tlsp_tls_mand_proto :
 			    var_tlsp_tls_proto,
 			    ask_ccert = ask_client_cert,
-			    fpt_dgst = var_tlsp_tls_fpt_dgst);
+			    mdalg = var_tlsp_tls_fpt_dgst);
     else
 	msg_warn("No server certs available. TLS can't be enabled");
 
@@ -1032,9 +1040,7 @@ int     main(int argc, char **argv)
 	0,
     };
     static const CONFIG_TIME_TABLE time_table[] = {
-	VAR_SMTPD_TLS_SCACHTIME, DEF_SMTPD_TLS_SCACHTIME, &var_smtpd_tls_scache_timeout, 0, 0,
 	VAR_TLSP_WATCHDOG, DEF_TLSP_WATCHDOG, &var_tlsp_watchdog, 10, 0,
-	VAR_TLSP_TLS_SCACHTIME, DEF_TLSP_TLS_SCACHTIME, &var_tlsp_tls_scache_timeout, 0, 0,
 	0,
     };
     static const CONFIG_BOOL_TABLE bool_table[] = {
@@ -1106,16 +1112,16 @@ int     main(int argc, char **argv)
      * Pass control to the single-threaded service skeleton.
      */
     event_server_main(argc, argv, tlsp_service,
-		      MAIL_SERVER_INT_TABLE, int_table,
-		      MAIL_SERVER_NINT_TABLE, nint_table,
-		      MAIL_SERVER_STR_TABLE, str_table,
-		      MAIL_SERVER_BOOL_TABLE, bool_table,
-		      MAIL_SERVER_NBOOL_TABLE, nbool_table,
-		      MAIL_SERVER_TIME_TABLE, time_table,
-		      MAIL_SERVER_PRE_INIT, pre_jail_init,
-		      MAIL_SERVER_POST_INIT, post_jail_init,
-		      MAIL_SERVER_SLOW_EXIT, tlsp_drain,
-		      MAIL_SERVER_WATCHDOG, &var_tlsp_watchdog,
+		      CA_MAIL_SERVER_INT_TABLE(int_table),
+		      CA_MAIL_SERVER_NINT_TABLE(nint_table),
+		      CA_MAIL_SERVER_STR_TABLE(str_table),
+		      CA_MAIL_SERVER_BOOL_TABLE(bool_table),
+		      CA_MAIL_SERVER_NBOOL_TABLE(nbool_table),
+		      CA_MAIL_SERVER_TIME_TABLE(time_table),
+		      CA_MAIL_SERVER_PRE_INIT(pre_jail_init),
+		      CA_MAIL_SERVER_POST_INIT(post_jail_init),
+		      CA_MAIL_SERVER_SLOW_EXIT(tlsp_drain),
+		      CA_MAIL_SERVER_WATCHDOG(&var_tlsp_watchdog),
 		      0);
 }
 
